/*
 * K-scope
 * Copyright 2012-2013 RIKEN, Japan
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.riken.kscope.xcodeml.xml;

import jp.riken.kscope.xcodeml.xml.gen.*;

/**
 * 式モデルインターフェイスクラス
 * @author riken
 *
 */
public interface IDefModelExpr {
	
	/**
	 * DivExpr(除算)要素を取得する
	 * @return    DivExpr(除算)要素
	 */
	public DivExpr getDivExpr();
	
	/**
	 * FarrayConstructor(配列構成子)要素を取得する
	 * @return		FarrayConstructor(配列構成子)要素
	 */
	public FarrayConstructor getFarrayConstructor();
	
	/**
	 * FarrayRef(部分配列、または配列要素参照)要素を取得する
	 * @return		FarrayRef(部分配列、または配列要素参照)要素
	 */
	public FarrayRef getFarrayRef();

	/**
	 * FcharacterConstant(文字列)要素を取得する
	 * @return		FcharacterConstant(文字列)要素
	 */
	public FcharacterConstant getFcharacterConstant();

	/**
	 * FcharacterRef(部分文字列参照)要素を取得する
	 * @return		FcharacterRef(部分文字列参照)要素
	 */
	public FcharacterRef getFcharacterRef();

	/**
	 * FcoArrayRef(coarray参照)要素を取得する
	 * @return		FcoArrayRef(coarray参照)要素
	 */
	public FcoArrayRef getFcoArrayRef();

	/**
	 * FcomplexConstant(COMPLEX型の定数)要素を取得する
	 * @return		FcomplexConstant(COMPLEX型の定数)要素
	 */
	public FcomplexConstant getFcomplexConstant();

	/**
	 * FconcatExpr(文字式の連結)要素を取得する
	 * @return		FconcatExpr(文字式の連結)要素
	 */
	public FconcatExpr getFconcatExpr();

	/**
	 * FdoLoop(DO型反復)要素を取得する
	 * @return		FdoLoop(DO型反復)要素
	 */
	public FdoLoop getFdoLoop();

	/**
	 * FintConstant(整数定数)要素を取得する
	 * @return		FintConstant(整数定数)要素
	 */
	public FintConstant getFintConstant();

	/**
	 * FlogicalConstant(論理値定数)要素を取得する
	 * @return		FlogicalConstant(論理値定数)要素
	 */
	public FlogicalConstant getFlogicalConstant();

	/**
	 * FmemberRef(構造体メンバ参照)要素を取得する
	 * @return		FmemberRef(構造体メンバ参照)要素
	 */
	public FmemberRef getFmemberRef();

	/**
	 * FpowerExpr(べき乗)要素を取得する
	 * @return		FpowerExpr(べき乗)要素
	 */
	public FpowerExpr getFpowerExpr();

	/**
	 * FrealConstant(浮動小数点数定数)要素を取得する
	 * @return		FrealConstant(浮動小数点数定数)要素
	 */
	public FrealConstant getFrealConstant();

	/**
	 * FstructConstructor(構造体構成子)要素を取得する
	 * @return		FstructConstructor(構造体構成子)要素
	 */
	public FstructConstructor getFstructConstructor();
	
	/**
	 * FunctionCall(関数・サブルーチン呼び出し)要素を取得する
	 * @return		FunctionCall(関数・サブルーチン呼び出し)要素
	 */
	public FunctionCall getFunctionCall();

	/**
	 * LogAndExpr(論理積)要素を取得する
	 * @return		LogAndExpr(論理積)要素
	 */
	public LogAndExpr getLogAndExpr();

	/**
	 * LogEQExpr(等価)要素を取得する
	 * @return		LogEQExpr(等価)要素
	 */
	public LogEQExpr getLogEQExpr();

	/**
	 * LogEQVExpr(論理等価)要素を取得する
	 * @return		LogEQVExpr(論理等価)要素
	 */
	public LogEQVExpr getLogEQVExpr();

	/**
	 * LogGEExpr(大なり、または同値)要素を取得する
	 * @return		LogGEExpr(大なり、または同値)要素
	 */
	public LogGEExpr getLogGEExpr();

	/**
	 * LogGTExpr(大なり)要素を取得する
	 * @return		LogGTExpr(大なり)要素
	 */
	public LogGTExpr getLogGTExpr();

	/**
	 * LogLEExpr(小なり、または等価)要素を取得する
	 * @return		LogLEExpr(小なり、または等価)要素
	 */
	public LogLEExpr getLogLEExpr();

	/**
	 * LogLTExpr(小なり)要素を取得する
	 * @return		LogLTExpr(小なり)要素
	 */
	public LogLTExpr getLogLTExpr();

	/**
	 * LogNEQExpr(非等価)要素を取得する
	 * @return		LogNEQExpr(非等価)要素
	 */
	public LogNEQExpr getLogNEQExpr();

	/**
	 * LogNEQVExpr(論理非等価)要素を取得する
	 * @return		LogNEQVExpr(論理非等価)要素
	 */
	public LogNEQVExpr getLogNEQVExpr();

	/**
	 * LogNotExpr(論理否定)要素を取得する
	 * @return		LogNotExpr(論理否定)要素
	 */
	public LogNotExpr getLogNotExpr();

	/**
	 * LogOrExpr(論理和)要素を取得する
	 * @return		LogOrExpr(論理和)要素
	 */
	public LogOrExpr getLogOrExpr();

	/**
	 * MinusExpr(減算)要素を取得する
	 * @return		MinusExpr(減算)要素
	 */
	public MinusExpr getMinusExpr();

	/**
	 * MulExpr(乗算)要素を取得する
	 * @return		MulExpr(乗算)要素
	 */
	public MulExpr getMulExpr();

	/**
	 * PlusExpr(加算)要素を取得する
	 * @return		PlusExpr(加算)要素
	 */
	public PlusExpr getPlusExpr();

	/**
	 * UnaryMinusExpr(符号反転)要素を取得する
	 * @return		UnaryMinusExpr(符号反転)要素
	 */
	public UnaryMinusExpr getUnaryMinusExpr();

	/**
	 * UserBinaryExpr(INTERFACE依存２項演算子)要素を取得する
	 * @return		UserBinaryExpr(INTERFACE依存２項演算子)要素
	 */
	public UserBinaryExpr getUserBinaryExpr();

	/**
	 * UserUnaryExpr(INTERFACE依存単項演算子)要素を取得する
	 * @return		UserUnaryExpr(INTERFACE依存単項演算子)要素
	 */
	public UserUnaryExpr getUserUnaryExpr();

	/**
	 * Var(変数名)要素を取得する
	 * @return		Var(変数名)要素
	 */
	public Var getVar();

	/**
	 * VarRef(変数参照)要素を取得する
	 * @return		VarRef(変数参照)要素
	 */
	public VarRef getVarRef();

	/**
	 * DivExpr(除算)要素を設定する
	 * @param  value    DivExpr(除算)要素
	 */
	public void setDivExpr(DivExpr value);

	/**
	 * FarrayConstructor(配列構成子)要素を取得する
	 * @param  value		FarrayConstructor(配列構成子)
	 */
	public void setFarrayConstructor(FarrayConstructor value);

	/**
	 * FarrayRef(部分配列、または配列要素参照)要素を取得する
	 * @param  value		FarrayRef(部分配列、または配列要素参照)要素
	 */
	public void setFarrayRef(FarrayRef value);

	/**
	 * FcharacterConstant(文字列)要素を取得する
	 * @param  value		FcharacterConstant(文字列)要素
	 */
	public void setFcharacterConstant(FcharacterConstant value);

	/**
	 * FcharacterRef(部分文字列参照)要素を取得する
	 * @param  value		FcharacterRef(部分文字列参照)要素
	 */
	public void setFcharacterRef(FcharacterRef value);

	/**
	 * FcoArrayRef(coarray参照)要素を取得する
	 * @param  value		FcoArrayRef(coarray参照)要素
	 */
	public void setFcoArrayRef(FcoArrayRef value);

	/**
	 * FcomplexConstant(COMPLEX型の定数)要素を取得する
	 * @param  value		FcomplexConstant(COMPLEX型の定数)要素
	 */
	public void setFcomplexConstant(FcomplexConstant value);

	/**
	 * FconcatExpr(文字式の連結)要素を取得する
	 * @param  value		FconcatExpr(文字式の連結)要素
	 */
	public void setFconcatExpr(FconcatExpr value);

	/**
	 * FdoLoop(DO型反復)要素を取得する
	 * @param  value		FdoLoop(DO型反復)要素
	 */
	public void setFdoLoop(FdoLoop value);

	/**
	 * FintConstant(整数定数)要素を取得する
	 * @param  value		FintConstant(整数定数)要素
	 */
	public void setFintConstant(FintConstant value);

	/**
	 * FlogicalConstant(論理値定数)要素を取得する
	 * @param  value		FlogicalConstant(論理値定数)要素
	 */
	public void setFlogicalConstant(FlogicalConstant value);

	/**
	 * FmemberRef(構造体メンバ参照)要素を取得する
	 * @param  value		FmemberRef(構造体メンバ参照)要素
	 */
	public void setFmemberRef(FmemberRef value);

	/**
	 * FpowerExpr(べき乗)要素を取得する
	 * @param  value		FpowerExpr(べき乗)要素
	 */
	public void setFpowerExpr(FpowerExpr value);

	/**
	 * FrealConstant(浮動小数点数定数)要素を取得する
	 * @param  value		FrealConstant(浮動小数点数定数)要素
	 */
	public void setFrealConstant(FrealConstant value);

	/**
	 * FstructConstructor(構造体構成子)要素を取得する
	 * @param  value		FstructConstructor(構造体構成子)要素
	 */
	public void setFstructConstructor(FstructConstructor value);

	/**
	 * FunctionCall(関数・サブルーチン呼び出し)要素を取得する
	 * @param  value		FunctionCall(関数・サブルーチン呼び出し)要素
	 */
	public void setFunctionCall(FunctionCall value);

	/**
	 * LogAndExpr(論理積)要素を取得する
	 * @param  value		LogAndExpr(論理積)要素
	 */
	public void setLogAndExpr(LogAndExpr value);

	/**
	 * LogEQExpr(等価)要素を取得する
	 * @param  value		LogEQExpr(等価)要素
	 */
	public void setLogEQExpr(LogEQExpr value);

	/**
	 * LogEQVExpr(論理等価)要素を取得する
	 * @param  value		LogEQVExpr(論理等価)要素
	 */
	public void setLogEQVExpr(LogEQVExpr value);

	/**
	 * LogGEExpr(大なり、または同値)要素を取得する
	 * @param  value		LogGEExpr(大なり、または同値)要素
	 */
	public void setLogGEExpr(LogGEExpr value);

	/**
	 * LogGTExpr(大なり)要素を取得する
	 * @param  value		LogGTExpr(大なり)要素
	 */
	public void setLogGTExpr(LogGTExpr value);

	/**
	 * LogLEExpr(小なり、または等価)要素を取得する
	 * @param  value		LogLEExpr(小なり、または等価)要素
	 */
	public void setLogLEExpr(LogLEExpr value);

	/**
	 * LogLTExpr(小なり)要素を取得する
	 * @param  value		LogLTExpr(小なり)要素
	 */
	public void setLogLTExpr(LogLTExpr value);

	/**
	 * LogNEQExpr(非等価)要素を取得する
	 * @param  value		LogNEQExpr(非等価)要素
	 */
	public void setLogNEQExpr(LogNEQExpr value);

	/**
	 * LogNEQVExpr(論理非等価)要素を取得する
	 * @param  value		LogNEQVExpr(論理非等価)要素
	 */
	public void setLogNEQVExpr(LogNEQVExpr value);

	/**
	 * LogNotExpr(論理否定)要素を取得する
	 * @param  value		LogNotExpr(論理否定)要素
	 */
	public void setLogNotExpr(LogNotExpr value);

	/**
	 * LogOrExpr(論理和)要素を取得する
	 * @param  value		LogOrExpr(論理和)要素
	 */
	public void setLogOrExpr(LogOrExpr value);

	/**
	 * MinusExpr(減算)要素を取得する
	 * @param  value		MinusExpr(減算)要素
	 */
	public void setMinusExpr(MinusExpr value);

	/**
	 * MulExpr(乗算)要素を取得する
	 * @param  value		MulExpr(乗算)要素
	 */
	public void setMulExpr(MulExpr value);

	/**
	 * PlusExpr(加算)要素を取得する
	 * @param  value		PlusExpr(加算)要素
	 */
	public void setPlusExpr(PlusExpr value);

	/**
	 * UnaryMinusExpr(符号反転)要素を取得する
	 * @param  value		UnaryMinusExpr(符号反転)要素
	 */
	public void setUnaryMinusExpr(UnaryMinusExpr value);

	/**
	 * UserBinaryExpr(INTERFACE依存２項演算子)要素を取得する
	 * @param  value		UserBinaryExpr(INTERFACE依存２項演算子)要素
	 */
	public void setUserBinaryExpr(UserBinaryExpr value);

	/**
	 * UserUnaryExpr(INTERFACE依存単項演算子)要素を取得する
	 * @param  value		UserUnaryExpr(INTERFACE依存単項演算子)要素
	 */
	public void setUserUnaryExpr(UserUnaryExpr value);

	/**
	 * Var(変数名)要素を取得する
	 * @param  value		Var(変数名)要素
	 */
	public void setVar(Var value);

	/**
	 * VarRef(変数参照)要素を取得する
	 * @param  value		VarRef(変数参照)要素
	 */
	public void setVarRef(VarRef value);
}
